/**
*****************************************************************************
**  File		: Driver_Keyboard_Matrix_4x4.c
**
**  Created on	: Feb 12, 2016
**
**  Author		: Kirill
**
**  Abstract    :    4*4.
**  			    ,   
**  			  .
**
**  Functions   : - Initial_Key_Port() -   
**  			  - Keyboard_scan(&kbd_keycode, &kbd_keyflags) -   ,
**  			  												    
**  			  												      .
**
**  			  - Key_Actions (&kbd_keycode, &kbd_keyflags); -      ,
**  			  												      
**  			  												     .
*****************************************************************************
*/
//---------------------------------------------------------
/* Includes */
#include "stm32f10x.h"
#include "Driver_Keyboard_Matrix_4x4.h"


/* Private macro */
//---------------------------------------------------------

/* Private variables */


//---------------------------------------------------------
/* Private function prototypes */


//---------------------------------------------------------
/* Private functions */
//=================================================================================================
void Initial_Key_Port(void)
{
	RCC->APB2ENR |= RCC_APB2Periph_GPIOA;		//<!   
	/*	//<!       
	 * K0 - K3 -  (,  ) - OD ( )
	 * K4 - K7 -  (,  ) - PU ( )
	 */
	GPIOA->CRL &= ~(GPIO_CRL_CNF0 | GPIO_CRL_CNF1 | GPIO_CRL_CNF2 | GPIO_CRL_CNF3);	//CNF 0 0
	GPIOA->CRL |= GPIO_CRL_CNF0_1 | GPIO_CRL_CNF1_1 | GPIO_CRL_CNF2_1 | GPIO_CRL_CNF3_1	| //CNF 1 0 (PU)
				  GPIO_CRL_MODE4_1 | GPIO_CRL_MODE5_1 | GPIO_CRL_MODE6_1 | GPIO_CRL_MODE7_1 ;//MODE 1 0, CNF 0 1 (OD)
	GPIOA->BSRR = KBD_K0 | KBD_K1 | KBD_K2 | KBD_K3 |	//PU
				  KBD_K4 | KBD_K5 | KBD_K6 | KBD_K7;	//Z  OD

	kbd_keycode = 0;			// 
	kbd_keyflags = 0;			//
}



//=================================================================================================
/*
 *    .
 * :    ,  ,
 *  ,   .
 * : kbd_keycode -   
 * 			  kbd_keyflags -    
 */
void Keyboard_scan (uint8_t *kbd_keycode, uint8_t *kbd_keyflags)
{
	/*
	 *       ( .0),
	 *         
	 *   .0 ( ),    detector,
	 *  detector !=0,    .
	 *    
	 * keypress -  
	 * keylongpress -   ( )
	 * keyrelease -       
	 * keycode -   ,   
	 */
	uint8_t detector;
	uint8_t accumulator = 0;
#ifdef USE_LONGPRESS
	static uint8_t keyhold_counter = 0;
#endif

	for (uint8_t row = 0; row < NUM_ROW; row++)
	{	/*       */
		switch (row)
		{
		case 0: KBD_ROW_PORT->BSRR = SEL_ROW_1; //   1
			break;
		case 1: KBD_ROW_PORT->BSRR = SEL_ROW_2; //   2
			break;
#if NUM_ROW > 2
		case 2: KBD_ROW_PORT->BSRR = SEL_ROW_3; //   3
			break;
#endif

#if NUM_ROW > 3
		case 3: KBD_ROW_PORT->BSRR = SEL_ROW_4; //   4
			break;
#endif

#if NUM_ROW > 4
		case 4: KBD_ROW_PORT->BSRR = SEL_ROW_5; //   5
			break;
#endif
		}
		//,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
		for (uint8_t col = 0; col < NUM_COL; col++)
		{ /*      
		  *   detector     */
			switch (col)
			{
			case 0: detector = ~(KBD_COL_PORT->IDR) & KBD_K7;	//  1
				break;
			case 1: detector = ~(KBD_COL_PORT->IDR) & KBD_K6;	//  2
				break;
#if NUM_COL > 2
			case 2: detector = ~(KBD_COL_PORT->IDR) & KBD_K5;	//  3
				break;
#endif

#if NUM_ROW > 3
			case 3: detector = ~(KBD_COL_PORT->IDR) & KBD_K4;	//  4
				break;
#endif

#if NUM_ROW > 4
			case 4: detector = ~(KBD_COL_PORT->IDR) & KBD_K5;	//  5
				break;
#endif
			}
			//........................................
			if (detector != 0)
			{ 	//  ,
				if ((*kbd_keyflags & KEYPRESS_FLAG) == 0)
				{ 	//       , 
					*kbd_keyflags |= KEYPRESS_FLAG;			//  
					*kbd_keycode = NUM_ROW*row+col+1;		//   
#ifdef USE_LONGPRESS
					keyhold_counter = 0;					//    
#endif
				}
#ifdef USE_LONGPRESS
				else
				{ //,       ,
					keyhold_counter++;						//.   
				}
				//..............
				if (keyhold_counter >= KEYHOLD_THRES)
				{ //      ,
					(*kbd_keyflags) |= KEYLONGPRESS_FLAG;	//   
				}
#endif
				//...............
			} else
			{ //,    , ,     
				accumulator++;
				if ((accumulator == NUM_ROW * NUM_COL) && ((*kbd_keyflags & KEYPRESS_FLAG) != 0))
				{
					*kbd_keyflags &= ~(KEYPRESS_FLAG | KEYLONGPRESS_FLAG);	//    
					*kbd_keyflags |= KEYRELEASE_FLAG;		//   
				}
			}
			//........................................
		}//,,,,,    () ,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	}//    ()
}//  

//=================================================================================================

/*
 *        .
 *         .
 *       .
 */
void Key_Actions (uint8_t *kbd_keycode, uint8_t *kbd_keyflags)
{
	if ((*kbd_keyflags & KEYPRESS_FLAG) != 0)
	{ //    
		if ((*kbd_keyflags & ACTION_COMPLETE) == 0)
		{//      
			switch (*kbd_keycode)
			{ //   
			case 1:			//    1
				//
				break;

			case 2:			//    2
				//
				break;

			case 3:			//    3
				//
				break;

			case 4:			//    4
				//
				break;

			case 5:			//    5
				//
				break;
			// ......      
			}
			*kbd_keyflags |= ACTION_COMPLETE;	//   
		}
	} else
	{ // ,
		if ((*kbd_keyflags & KEYRELEASE_FLAG) != 0)
		{ //     ,
			*kbd_keyflags &= ~ACTION_COMPLETE;	//   
		}
	}
}
//=================================================================================================

